@@ -22,6 +22,7 @@ limitations under the License.
 package support
 
 import (
+	"bytes"
 	"bufio"
 	"context"
 	"encoding/json"
@@ -50,8 +51,8 @@ import (
 
 	ctrl "sigs.k8s.io/controller-runtime/pkg/client"
 
-	eventing "knative.dev/eventing/pkg/apis/eventing/v1beta1"
-	messaging "knative.dev/eventing/pkg/apis/messaging/v1beta1"
+	eventing "knative.dev/eventing/pkg/apis/eventing/v1"
+	messaging "knative.dev/eventing/pkg/apis/messaging/v1"
 	servingv1 "knative.dev/serving/pkg/apis/serving/v1"
 
 	projectv1 "github.com/openshift/api/project/v1"
@@ -266,7 +267,7 @@ func Logs(ns, podName string, options corev1.PodLogOptions) func() string {
 	}
 }
 
-func StructuredLogs(ns, podName string, options corev1.PodLogOptions) []util.LogEntry {
+func StructuredLogs(ns, podName string, options corev1.PodLogOptions, ignoreParseErrors bool) []util.LogEntry {
 	byteReader, err := TestClient().CoreV1().Pods(ns).GetLogs(podName, &options).Stream(TestContext)
 	if err != nil {
 		log.Error(err, "Error while reading container logs")
@@ -285,8 +286,14 @@ func StructuredLogs(ns, podName string, options corev1.PodLogOptions) []util.Log
 		t := scanner.Text()
 		err := json.Unmarshal([]byte(t), &entry)
 		if err != nil {
-			continue
+			if ignoreParseErrors {
+				continue
+			} else {
+				log.Errorf(err, "Unable to parse structured content: %s", t)
+				return nil
+			}
 		}
+
 		entries = append(entries, entry)
 	}
 
@@ -653,6 +660,51 @@ func Configmap(ns string, name string) func() *corev1.ConfigMap {
 	}
 }
 
+func NewPlainTextConfigmap(ns string, name string, data map[string]string) error {
+	cm := corev1.ConfigMap{
+		TypeMeta: metav1.TypeMeta{
+			Kind:       "ConfigMap",
+			APIVersion: corev1.SchemeGroupVersion.String(),
+		},
+		ObjectMeta: metav1.ObjectMeta{
+			Namespace: ns,
+			Name:      name,
+		},
+		Data: data,
+	}
+	return TestClient().Create(TestContext, &cm)
+}
+
+func NewBinaryConfigmap(ns string, name string, data map[string][]byte) error {
+	cm := corev1.ConfigMap{
+		TypeMeta: metav1.TypeMeta{
+			Kind:       "ConfigMap",
+			APIVersion: corev1.SchemeGroupVersion.String(),
+		},
+		ObjectMeta: metav1.ObjectMeta{
+			Namespace: ns,
+			Name:      name,
+		},
+		BinaryData: data,
+	}
+	return TestClient().Create(TestContext, &cm)
+}
+
+func NewPlainTextSecret(ns string, name string, data map[string]string) error {
+	sec := corev1.Secret{
+		TypeMeta: metav1.TypeMeta{
+			Kind:       "Secret",
+			APIVersion: corev1.SchemeGroupVersion.String(),
+		},
+		ObjectMeta: metav1.ObjectMeta{
+			Namespace: ns,
+			Name:      name,
+		},
+		StringData: data,
+	}
+	return TestClient().Create(TestContext, &sec)
+}
+
 func KnativeService(ns string, name string) func() *servingv1.Service {
 	return func() *servingv1.Service {
 		answer := servingv1.Service{
@@ -705,6 +757,25 @@ func Deployment(ns string, name string) func() *appsv1.Deployment {
 	}
 }
 
+func DeploymentCondition(ns string, name string, conditionType appsv1.DeploymentConditionType) func() appsv1.DeploymentCondition {
+	return func() appsv1.DeploymentCondition {
+		deployment := Deployment(ns, name)()
+
+		condition := appsv1.DeploymentCondition{
+			Status: corev1.ConditionUnknown,
+		}
+
+		for _, c := range deployment.Status.Conditions {
+			if c.Type == conditionType {
+				condition = c
+				break
+			}
+		}
+
+		return condition
+	}
+}
+
 func Build(ns string, name string) func() *v1.Build {
 	return func() *v1.Build {
 		build := v1.NewBuild(ns, name)
@@ -816,35 +887,6 @@ func OperatorPod(ns string) func() *corev1.Pod {
 	}
 }
 
-func OperatorTryPodForceKill(ns string, timeSeconds int) {
-	pod := OperatorPod(ns)()
-	if pod != nil {
-		if err := TestClient().Delete(TestContext, pod, ctrl.GracePeriodSeconds(timeSeconds)); err != nil {
-			log.Error(err, "cannot forcefully kill the pod")
-		}
-	}
-}
-
-func ScaleOperator(ns string, replicas int32) func() error {
-	return func() error {
-		operator, err := TestClient().AppsV1().Deployments(ns).Get(TestContext, "camel-k-operator", metav1.GetOptions{})
-		if err != nil {
-			return err
-		}
-		operator.Spec.Replicas = &replicas
-		_, err = TestClient().AppsV1().Deployments(ns).Update(TestContext, operator, metav1.UpdateOptions{})
-		if err != nil {
-			return err
-		}
-
-		if replicas == 0 {
-			// speedup scale down by killing the pod
-			OperatorTryPodForceKill(ns, 10)
-		}
-		return nil
-	}
-}
-
 func Role(ns string) func() *rbacv1.Role {
 	return func() *rbacv1.Role {
 		lst := rbacv1.RoleList{
@@ -985,23 +1027,7 @@ func CreateKamelPod(ns string, name string, command ...string) error {
 	Knative
 */
 
-func CreateKnativeChannelv1Alpha1(ns string, name string) func() error {
-	return func() error {
-		channel := messaging.InMemoryChannel{
-			TypeMeta: metav1.TypeMeta{
-				Kind:       "InMemoryChannel",
-				APIVersion: messaging.SchemeGroupVersion.String(),
-			},
-			ObjectMeta: metav1.ObjectMeta{
-				Namespace: ns,
-				Name:      name,
-			},
-		}
-		return TestClient().Create(TestContext, &channel)
-	}
-}
-
-func CreateKnativeChannelv1Beta1(ns string, name string) func() error {
+func CreateKnativeChannel(ns string, name string) func() error {
 	return func() error {
 		channel := messaging.InMemoryChannel{
 			TypeMeta: metav1.TypeMeta{
@@ -1301,3 +1327,21 @@ func NewTestNamespace(injectKnativeBroker bool) ctrl.Object {
 	}
 	return obj
 }
+
+func GetOutputString(command *cobra.Command) string {
+	var buf bytes.Buffer
+	reader, writer, err := os.Pipe()
+	if err != nil {
+		panic(err)
+	}
+
+	command.SetOut(writer)
+	command.Execute()
+
+	writer.Close()
+	defer reader.Close()
+
+	buf.ReadFrom(reader)
+
+	return buf.String()
+}
