@@ -25,49 +25,34 @@ import (
 // NOTE: json tags are required.  Any new fields you add must have json tags for the fields to be serialized.
 // Important: Run "make generate-deepcopy" to regenerate code after modifying this file
 
-// BuildSpec defines the desired state of Build
+// BuildSpec defines the Build to be executed
 type BuildSpec struct {
 	Tasks []Task `json:"tasks,omitempty"`
+	// Timeout defines the Build maximum execution duration.
+	// The Build deadline is set to the Build start time plus the Timeout duration.
+	// If the Build deadline is exceeded, the Build context is canceled,
+	// and its phase set to BuildPhaseFailed.
+	// +kubebuilder:validation:Format=duration
+	Timeout metav1.Duration `json:"timeout,omitempty"`
 }
 
 // Task --
 type Task struct {
-	Builder *BuilderTask `json:"builder,omitempty"`
-	Image   *ImageTask   `json:"image,omitempty"`
+	Builder  *BuilderTask  `json:"builder,omitempty"`
+	Buildah  *BuildahTask  `json:"buildah,omitempty"`
+	Kaniko   *KanikoTask   `json:"kaniko,omitempty"`
+	Spectrum *SpectrumTask `json:"spectrum,omitempty"`
+	S2i      *S2iTask      `json:"s2i,omitempty"`
 }
 
 // BaseTask --
 type BaseTask struct {
-	Name         string               `json:"name,omitempty"`
-	Affinity     *corev1.Affinity     `json:"affinity,omitempty"`
-	Volumes      []corev1.Volume      `json:"volumes,omitempty"`
-	VolumeMounts []corev1.VolumeMount `json:"volumeMounts,omitempty"`
-}
-
-// ContainerTask --
-type ContainerTask struct {
-	BaseTask        `json:",inline"`
-	Image           string                  `json:"image,omitempty"`
-	Command         []string                `json:"command,omitempty"`
-	Args            []string                `json:"args,omitempty"`
-	Env             []corev1.EnvVar         `json:"env,omitempty"`
-	WorkingDir      string                  `json:"workingDir,omitempty"`
-	SecurityContext *corev1.SecurityContext `json:"securityContext,omitempty"`
-}
-
-// ImageTask --
-type ImageTask struct {
-	ContainerTask `json:",inline"`
-	BuiltImage    string `json:"builtImage,omitempty"`
+	Name string `json:"name,omitempty"`
 }
 
 // BuilderTask --
 type BuilderTask struct {
-	BaseTask `json:",inline"`
-	// This is required until https://github.com/kubernetes-sigs/controller-tools/pull/395 gets merged
-	// +kubebuilder:pruning:PreserveUnknownFields
-	Meta         metav1.ObjectMeta `json:"meta,omitempty"`
-	Image        string            `json:"image,omitempty"`
+	BaseTask     `json:",inline"`
 	BaseImage    string            `json:"baseImage,omitempty"`
 	Runtime      RuntimeSpec       `json:"runtime,omitempty"`
 	Sources      []SourceSpec      `json:"sources,omitempty"`
@@ -76,8 +61,50 @@ type BuilderTask struct {
 	Steps        []string          `json:"steps,omitempty"`
 	Maven        MavenSpec         `json:"maven,omitempty"`
 	BuildDir     string            `json:"buildDir,omitempty"`
-	Properties   map[string]string `json:"properties,omitempty"`
-	Timeout      metav1.Duration   `json:"timeout,omitempty"`
+}
+
+// PublishTask --
+type PublishTask struct {
+	ContextDir string                          `json:"contextDir,omitempty"`
+	BaseImage  string                          `json:"baseImage,omitempty"`
+	Image      string                          `json:"image,omitempty"`
+	Registry   IntegrationPlatformRegistrySpec `json:"registry,omitempty"`
+}
+
+// BuildahTask --
+type BuildahTask struct {
+	BaseTask        `json:",inline"`
+	PublishTask     `json:",inline"`
+	Verbose         *bool  `json:"verbose,omitempty"`
+	HttpProxySecret string `json:"httpProxySecret,omitempty"`
+}
+
+// KanikoTask --
+type KanikoTask struct {
+	BaseTask        `json:",inline"`
+	PublishTask     `json:",inline"`
+	Verbose         *bool           `json:"verbose,omitempty"`
+	HttpProxySecret string          `json:"httpProxySecret,omitempty"`
+	Cache           KanikoTaskCache `json:"cache,omitempty"`
+}
+
+// KanikoTaskCache --
+type KanikoTaskCache struct {
+	Enabled               *bool  `json:"enabled,omitempty"`
+	PersistentVolumeClaim string `json:"persistentVolumeClaim,omitempty"`
+}
+
+// SpectrumTask --
+type SpectrumTask struct {
+	BaseTask    `json:",inline"`
+	PublishTask `json:",inline"`
+}
+
+// S2iTask --
+type S2iTask struct {
+	BaseTask   `json:",inline"`
+	ContextDir string `json:"contextDir,omitempty"`
+	Tag        string `json:"tag,omitempty"`
 }
 
 // BuildStatus defines the observed state of Build
