@@ -39,6 +39,7 @@ import (
 	corev1 "k8s.io/api/core/v1"
 	k8serrors "k8s.io/apimachinery/pkg/api/errors"
 	metav1 "k8s.io/apimachinery/pkg/apis/meta/v1"
+	"k8s.io/apimachinery/pkg/util/yaml"
 
 	ctrl "sigs.k8s.io/controller-runtime/pkg/client"
 
@@ -46,9 +47,10 @@ import (
 	"github.com/apache/camel-k/pkg/client"
 	"github.com/apache/camel-k/pkg/trait"
 	"github.com/apache/camel-k/pkg/util"
-	"github.com/apache/camel-k/pkg/util/flow"
+	"github.com/apache/camel-k/pkg/util/dsl"
 	"github.com/apache/camel-k/pkg/util/kubernetes"
 	k8slog "github.com/apache/camel-k/pkg/util/kubernetes/log"
+	"github.com/apache/camel-k/pkg/util/property"
 	"github.com/apache/camel-k/pkg/util/sync"
 	"github.com/apache/camel-k/pkg/util/watch"
 )
@@ -77,9 +79,12 @@ func newCmdRun(rootCmdOptions *RootCmdOptions) (*cobra.Command, *runCmdOptions)
 	cmd.Flags().StringArrayP("dependency", "d", nil, "A dependency that should be included, e.g., \"-d camel-mail\" for a Camel component, or \"-d mvn:org.my:app:1.0\" for a Maven dependency")
 	cmd.Flags().BoolP("wait", "w", false, "Wait for the integration to be running")
 	cmd.Flags().StringP("kit", "k", "", "The kit used to run the integration")
-	cmd.Flags().StringArrayP("property", "p", nil, "Add a camel property")
-	cmd.Flags().StringArray("configmap", nil, "Add a ConfigMap")
-	cmd.Flags().StringArray("secret", nil, "Add a Secret")
+	cmd.Flags().StringArrayP("property", "p", nil, "Add a runtime property or properties file (syntax: [my-key=my-value|file:/path/to/my-conf.properties])")
+	cmd.Flags().StringArray("build-property", nil, "Add a build time property or properties file (syntax: [my-key=my-value|file:/path/to/my-conf.properties])")
+	cmd.Flags().StringArray("config", nil, "Add a runtime configuration from a Configmap, a Secret or a file (syntax: [configmap|secret|file]:name[/key], where name represents the local file path or the configmap/secret name and key optionally represents the configmap/secret key to be filtered)")
+	cmd.Flags().StringArray("resource", nil, "Add a runtime resource from a Configmap, a Secret or a file (syntax: [configmap|secret|file]:name[/key][@path], where name represents the local file path or the configmap/secret name, key optionally represents the configmap/secret key to be filtered and path represents the destination path)")
+	cmd.Flags().StringArray("configmap", nil, "[Deprecated] Add a ConfigMap")
+	cmd.Flags().StringArray("secret", nil, "[Deprecated] Add a Secret")
 	cmd.Flags().StringArray("maven-repository", nil, "Add a maven repository")
 	cmd.Flags().Bool("logs", false, "Print integration logs")
 	cmd.Flags().Bool("sync", false, "Synchronize the local source file with the cluster, republishing at each change")
@@ -87,16 +92,15 @@ func newCmdRun(rootCmdOptions *RootCmdOptions) (*cobra.Command, *runCmdOptions)
 	cmd.Flags().Bool("use-flows", true, "Write yaml sources as Flow objects in the integration custom resource")
 	cmd.Flags().String("profile", "", "Trait profile used for deployment")
 	cmd.Flags().StringArrayP("trait", "t", nil, "Configure a trait. E.g. \"-t service.enabled=false\"")
-	cmd.Flags().StringArray("logging-level", nil, "Configure the logging level. e.g. \"--logging-level org.apache.camel=DEBUG\"")
 	cmd.Flags().StringP("output", "o", "", "Output format. One of: json|yaml")
 	cmd.Flags().Bool("compression", false, "Enable storage of sources and resources as a compressed binary blobs")
-	cmd.Flags().StringArray("resource", nil, "Add a resource")
 	cmd.Flags().StringArray("open-api", nil, "Add an OpenAPI v2 spec")
 	cmd.Flags().StringArrayP("volume", "v", nil, "Mount a volume into the integration container. E.g \"-v pvcname:/container/path\"")
 	cmd.Flags().StringArrayP("env", "e", nil, "Set an environment variable in the integration container. E.g \"-e MY_VAR=my-value\"")
-	cmd.Flags().StringArray("property-file", nil, "Bind a property file to the integration. E.g. \"--property-file integration.properties\"")
+	cmd.Flags().StringArray("property-file", nil, "[Deprecated] Bind a property file to the integration. E.g. \"--property-file integration.properties\"")
 	cmd.Flags().StringArray("label", nil, "Add a label to the integration. E.g. \"--label my.company=hello\"")
 	cmd.Flags().StringArray("source", nil, "Add source file to your integration, this is added to the list of files listed as arguments of the command")
+	cmd.Flags().String("pod-template", "", "The path of the YAML file containing a PodSpec template to be used for the Integration pods")
 
 	cmd.Flags().Bool("save", false, "Save the run parameters into the default kamel configuration file (kamel-config.yaml)")
 
@@ -119,21 +123,24 @@ type runCmdOptions struct {
 	IntegrationName string   `mapstructure:"name" yaml:",omitempty"`
 	Profile         string   `mapstructure:"profile" yaml:",omitempty"`
 	OutputFormat    string   `mapstructure:"output" yaml:",omitempty"`
+	PodTemplate     string   `mapstructure:"pod-template" yaml:",omitempty"`
 	Connects        []string `mapstructure:"connects" yaml:",omitempty"`
 	Resources       []string `mapstructure:"resources" yaml:",omitempty"`
 	OpenAPIs        []string `mapstructure:"open-apis" yaml:",omitempty"`
 	Dependencies    []string `mapstructure:"dependencies" yaml:",omitempty"`
 	Properties      []string `mapstructure:"properties" yaml:",omitempty"`
+	BuildProperties []string `mapstructure:"build-properties" yaml:",omitempty"`
+	Configs         []string `mapstructure:"configs" yaml:",omitempty"`
 	ConfigMaps      []string `mapstructure:"configmaps" yaml:",omitempty"`
 	Secrets         []string `mapstructure:"secrets" yaml:",omitempty"`
 	Repositories    []string `mapstructure:"maven-repositories" yaml:",omitempty"`
 	Traits          []string `mapstructure:"traits" yaml:",omitempty"`
-	LoggingLevels   []string `mapstructure:"logging-levels" yaml:",omitempty"`
 	Volumes         []string `mapstructure:"volumes" yaml:",omitempty"`
 	EnvVars         []string `mapstructure:"envs" yaml:",omitempty"`
-	PropertyFiles   []string `mapstructure:"property-files" yaml:",omitempty"`
-	Labels          []string `mapstructure:"labels" yaml:",omitempty"`
-	Sources         []string `mapstructure:"sources" yaml:",omitempty"`
+	// Deprecated: PropertyFiles has been deprecated in 1.5
+	PropertyFiles []string `mapstructure:"property-files" yaml:",omitempty"`
+	Labels        []string `mapstructure:"labels" yaml:",omitempty"`
+	Sources       []string `mapstructure:"sources" yaml:",omitempty"`
 }
 
 func (o *runCmdOptions) decode(cmd *cobra.Command, args []string) error {
@@ -219,11 +226,29 @@ func (o *runCmdOptions) validate() error {
 		}
 	}
 
+	// Deprecation warning
+	if o.PropertyFiles != nil {
+		fmt.Println("Warn: --property-file has been deprecated. You should use --property file:/path/to/conf.properties instead.")
+	}
+	if o.ConfigMaps != nil {
+		fmt.Println("Warn: --configmap has been deprecated. You should use --config configmap:my-configmap instead.")
+	}
+	if o.Secrets != nil {
+		fmt.Println("Warn: --secret has been deprecated. You should use --config secret:my-secret instead.")
+	}
+
 	err := validatePropertyFiles(o.PropertyFiles)
 	if err != nil {
 		return err
 	}
 
+	propertyFiles := filterBuildPropertyFiles(o.Properties)
+	propertyFiles = append(propertyFiles, filterBuildPropertyFiles(o.BuildProperties)...)
+	err = validatePropertyFiles(propertyFiles)
+	if err != nil {
+		return err
+	}
+
 	for _, label := range o.Labels {
 		parts := strings.Split(label, "=")
 		if len(parts) != 2 {
@@ -234,6 +259,17 @@ func (o *runCmdOptions) validate() error {
 	return nil
 }
 
+func filterBuildPropertyFiles(maybePropertyFiles []string) []string {
+	var propertyFiles []string
+	for _, maybePropertyFile := range maybePropertyFiles {
+		if strings.HasPrefix(maybePropertyFile, "file:") {
+			propertyFiles = append(propertyFiles, strings.Replace(maybePropertyFile, "file:", "", 1))
+		}
+	}
+
+	return propertyFiles
+}
+
 func (o *runCmdOptions) run(cmd *cobra.Command, args []string) error {
 	c, err := o.GetCmdClient()
 	if err != nil {
@@ -251,7 +287,7 @@ func (o *runCmdOptions) run(cmd *cobra.Command, args []string) error {
 		}
 	}
 
-	integration, err := o.createIntegration(c, args, catalog)
+	integration, err := o.createOrUpdateIntegration(c, args, catalog)
 	if err != nil {
 		return err
 	}
@@ -371,12 +407,19 @@ func (o *runCmdOptions) syncIntegration(cmd *cobra.Command, c client.Client, sou
 	// Let's watch all relevant files when in dev mode
 	var files []string
 	files = append(files, sources...)
-	files = append(files, o.Resources...)
+	files = append(files, filterFileLocation(o.Resources)...)
+	files = append(files, filterFileLocation(o.Configs)...)
+	files = append(files, filterFileLocation(o.Properties)...)
+	files = append(files, filterFileLocation(o.BuildProperties)...)
 	files = append(files, o.PropertyFiles...)
 	files = append(files, o.OpenAPIs...)
 
 	for _, s := range files {
-		if isLocalAndFileExists(s) {
+		ok, err := isLocalAndFileExists(s)
+		if err != nil {
+			return err
+		}
+		if ok {
 			changes, err := sync.File(o.Context, s)
 			if err != nil {
 				return err
@@ -398,7 +441,7 @@ func (o *runCmdOptions) syncIntegration(cmd *cobra.Command, c client.Client, sou
 						newCmd.Args = o.validateArgs
 						newCmd.PreRunE = o.decode
 						newCmd.RunE = func(cmd *cobra.Command, args []string) error {
-							_, err := o.updateIntegrationCode(c, sources, catalog)
+							_, err := o.createOrUpdateIntegration(c, sources, catalog)
 							return err
 						}
 						newCmd.PostRunE = nil
@@ -421,20 +464,36 @@ func (o *runCmdOptions) syncIntegration(cmd *cobra.Command, c client.Client, sou
 	return nil
 }
 
-func (o *runCmdOptions) createIntegration(c client.Client, sources []string, catalog *trait.Catalog) (*v1.Integration, error) {
-	return o.updateIntegrationCode(c, sources, catalog)
-}
-
 // nolint: gocyclo
-func (o *runCmdOptions) updateIntegrationCode(c client.Client, sources []string, catalog *trait.Catalog) (*v1.Integration, error) {
+func (o *runCmdOptions) createOrUpdateIntegration(c client.Client, sources []string, catalog *trait.Catalog) (*v1.Integration, error) {
 	namespace := o.Namespace
-
 	name := o.GetIntegrationName(sources)
 
 	if name == "" {
 		return nil, errors.New("unable to determine integration name")
 	}
 
+	integration := &v1.Integration{
+		TypeMeta: metav1.TypeMeta{
+			Kind:       v1.IntegrationKind,
+			APIVersion: v1.SchemeGroupVersion.String(),
+		},
+		ObjectMeta: metav1.ObjectMeta{
+			Namespace: namespace,
+			Name:      name,
+		},
+	}
+
+	existing := &v1.Integration{}
+	err := c.Get(o.Context, ctrl.ObjectKeyFromObject(integration), existing)
+	if err == nil {
+		integration = existing.DeepCopy()
+	} else if k8serrors.IsNotFound(err) {
+		existing = nil
+	} else {
+		return nil, err
+	}
+
 	var integrationKit *corev1.ObjectReference
 	if o.IntegrationKit != "" {
 		integrationKit = &corev1.ObjectReference{
@@ -443,22 +502,12 @@ func (o *runCmdOptions) updateIntegrationCode(c client.Client, sources []string,
 		}
 	}
 
-	integration := v1.Integration{
-		TypeMeta: metav1.TypeMeta{
-			Kind:       v1.IntegrationKind,
-			APIVersion: v1.SchemeGroupVersion.String(),
-		},
-		ObjectMeta: metav1.ObjectMeta{
-			Namespace: namespace,
-			Name:      name,
-		},
-		Spec: v1.IntegrationSpec{
-			Dependencies:   make([]string, 0, len(o.Dependencies)),
-			IntegrationKit: integrationKit,
-			Configuration:  make([]v1.ConfigurationSpec, 0),
-			Repositories:   o.Repositories,
-			Profile:        v1.TraitProfileByName(o.Profile),
-		},
+	integration.Spec = v1.IntegrationSpec{
+		Dependencies:   make([]string, 0, len(o.Dependencies)),
+		IntegrationKit: integrationKit,
+		Configuration:  make([]v1.ConfigurationSpec, 0),
+		Repositories:   o.Repositories,
+		Profile:        v1.TraitProfileByName(o.Profile),
 	}
 
 	for _, label := range o.Labels {
@@ -482,7 +531,7 @@ func (o *runCmdOptions) updateIntegrationCode(c client.Client, sources []string,
 
 	for _, source := range resolvedSources {
 		if o.UseFlows && !o.Compression && (strings.HasSuffix(source.Name, ".yaml") || strings.HasSuffix(source.Name, ".yml")) {
-			flows, err := flow.FromYamlDSLString(source.Content)
+			flows, err := dsl.FromYamlDSLString(source.Content)
 			if err != nil {
 				return nil, err
 			}
@@ -498,48 +547,70 @@ func (o *runCmdOptions) updateIntegrationCode(c client.Client, sources []string,
 		}
 	}
 
-	for _, resource := range o.Resources {
-		rawData, contentType, err := loadRawContent(resource)
-		if err != nil {
-			return nil, err
-		}
+	err = resolvePodTemplate(context.Background(), o.PodTemplate, &integration.Spec)
+	if err != nil {
+		return nil, err
+	}
 
-		resourceSpec, err := binaryOrTextResource(path.Base(resource), rawData, contentType, o.Compression)
-		if err != nil {
-			return nil, err
+	for _, resource := range o.Resources {
+		if config, parseErr := ParseResourceOption(resource); parseErr == nil {
+			if applyResourceOptionErr := ApplyResourceOption(config, &integration.Spec, c, namespace, o.Compression); applyResourceOptionErr != nil {
+				return nil, applyResourceOptionErr
+			}
+		} else {
+			return nil, parseErr
 		}
-		integration.Spec.AddResources(resourceSpec)
 	}
 
 	for _, resource := range o.OpenAPIs {
-		data, _, compressed, err := loadTextContent(resource, o.Compression)
-		if err != nil {
+		if err = addResource(resource, &integration.Spec, o.Compression, v1.ResourceTypeOpenAPI); err != nil {
 			return nil, err
 		}
-
-		integration.Spec.AddResources(v1.ResourceSpec{
-			DataSpec: v1.DataSpec{
-				Name:        path.Base(resource),
-				Content:     data,
-				Compression: compressed,
-			},
-			Type: v1.ResourceTypeOpenAPI,
-		})
 	}
 
 	for _, item := range o.Dependencies {
 		integration.Spec.AddDependency(item)
 	}
-	for _, pf := range o.PropertyFiles {
-		if err := addPropertyFile(pf, &integration.Spec); err != nil {
+	for _, item := range o.PropertyFiles {
+		// Deprecated: making it compatible with newer mechanism
+		o.Properties = append(o.Properties, "file:"+item)
+	}
+	for _, item := range o.Properties {
+		props, err := extractProperties(item)
+		if err != nil {
+			return nil, err
+		}
+		if err := addIntegrationProperties(props, &integration.Spec); err != nil {
 			return nil, err
 		}
 	}
-	for _, item := range o.Properties {
-		integration.Spec.AddConfiguration("property", item)
+	// convert each build configuration to a builder trait property
+	for _, item := range o.BuildProperties {
+		props, err := extractProperties(item)
+		if err != nil {
+			return nil, err
+		}
+		for _, k := range props.Keys() {
+			v, ok := props.Get(k)
+			if ok {
+				entry, err := property.EncodePropertyFileEntry(k, v)
+				if err != nil {
+					return nil, err
+				}
+				o.Traits = append(o.Traits, fmt.Sprintf("builder.properties=%s", entry))
+			} else {
+				return nil, err
+			}
+		}
 	}
-	for _, item := range o.LoggingLevels {
-		integration.Spec.AddConfiguration("property", "logging.level."+item)
+	for _, item := range o.Configs {
+		if config, parseErr := ParseConfigOption(item); parseErr == nil {
+			if applyConfigOptionErr := ApplyConfigOption(config, &integration.Spec, c, namespace, o.Compression); applyConfigOptionErr != nil {
+				return nil, applyConfigOptionErr
+			}
+		} else {
+			return nil, parseErr
+		}
 	}
 	for _, item := range o.ConfigMaps {
 		integration.Spec.AddConfiguration("configmap", item)
@@ -554,7 +625,7 @@ func (o *runCmdOptions) updateIntegrationCode(c client.Client, sources []string,
 		integration.Spec.AddConfiguration("env", item)
 	}
 
-	if err := o.configureTraits(&integration, o.Traits, catalog); err != nil {
+	if err := o.configureTraits(integration, o.Traits, catalog); err != nil {
 		return nil, err
 	}
 
@@ -562,7 +633,7 @@ func (o *runCmdOptions) updateIntegrationCode(c client.Client, sources []string,
 	case "":
 		// continue..
 	case "yaml":
-		data, err := kubernetes.ToYAML(&integration)
+		data, err := kubernetes.ToYAML(integration)
 		if err != nil {
 			return nil, err
 		}
@@ -570,7 +641,7 @@ func (o *runCmdOptions) updateIntegrationCode(c client.Client, sources []string,
 		return nil, nil
 
 	case "json":
-		data, err := kubernetes.ToJSON(&integration)
+		data, err := kubernetes.ToJSON(integration)
 		if err != nil {
 			return nil, err
 		}
@@ -581,76 +652,53 @@ func (o *runCmdOptions) updateIntegrationCode(c client.Client, sources []string,
 		return nil, fmt.Errorf("invalid output format option '%s', should be one of: yaml|json", o.OutputFormat)
 	}
 
-	existed := false
-	err = c.Create(o.Context, &integration)
-	if err != nil && k8serrors.IsAlreadyExists(err) {
-		existed = true
-		existing := &v1.Integration{
-			ObjectMeta: metav1.ObjectMeta{
-				Namespace: integration.Namespace,
-				Name:      integration.Name,
-			},
-		}
-		err = c.Get(o.Context, ctrl.ObjectKeyFromObject(existing), existing)
-		if err != nil {
-			return nil, err
-		}
-		// Hold the resource from the operator controller
-		existing.Status.Phase = v1.IntegrationPhaseUpdating
-		err = c.Status().Update(o.Context, existing)
-		if err != nil {
-			return nil, err
-		}
-		// Update the spec
-		integration.ResourceVersion = existing.ResourceVersion
-		err = c.Update(o.Context, &integration)
-		if err != nil {
-			return nil, err
-		}
-		// Reset the status
-		integration.Status = v1.IntegrationStatus{}
-		err = c.Status().Update(o.Context, &integration)
+	if existing == nil {
+		err = c.Create(o.Context, integration)
+	} else {
+		err = c.Patch(o.Context, integration, ctrl.MergeFromWithOptions(existing, ctrl.MergeFromWithOptimisticLock{}))
 	}
 
 	if err != nil {
 		return nil, err
 	}
 
-	if !existed {
-		fmt.Printf("integration \"%s\" created\n", name)
+	if existing == nil {
+		fmt.Printf("Integration \"%s\" created\n", name)
 	} else {
-		fmt.Printf("integration \"%s\" updated\n", name)
+		fmt.Printf("Integration \"%s\" updated\n", name)
 	}
-	return &integration, nil
+	return integration, nil
 }
 
-func binaryOrTextResource(fileName string, data []byte, contentType string, base64Compression bool) (v1.ResourceSpec, error) {
-	resourceSpec := v1.ResourceSpec{
-		DataSpec: v1.DataSpec{
-			Name:        fileName,
-			ContentKey:  fileName,
-			ContentType: contentType,
-			Compression: false,
-		},
-		Type: v1.ResourceTypeData,
+func addResource(resourceLocation string, integrationSpec *v1.IntegrationSpec, enableCompression bool, resourceType v1.ResourceType) error {
+	if data, _, compressed, err := loadTextContent(resourceLocation, enableCompression); err == nil {
+		integrationSpec.AddResources(v1.ResourceSpec{
+			DataSpec: v1.DataSpec{
+				Name:        path.Base(resourceLocation),
+				Content:     data,
+				Compression: compressed,
+			},
+			Type: resourceType,
+		})
+	} else {
+		return err
 	}
 
-	if !base64Compression && isBinary(contentType) {
-		resourceSpec.RawContent = data
-		return resourceSpec, nil
+	return nil
+}
+
+// The function parse the value and if it is a file (file:/path/), it will parse as property file
+// otherwise return a single property built from the item passed as `key=value`
+func extractProperties(value string) (*properties.Properties, error) {
+	if !strings.HasPrefix(value, "file:") {
+		return keyValueProps(value)
 	}
-	// either is a text resource or base64 compression is enabled
-	if base64Compression {
-		content, err := compressToString(data)
-		if err != nil {
-			return resourceSpec, err
-		}
-		resourceSpec.Content = content
-		resourceSpec.Compression = true
-	} else {
-		resourceSpec.Content = string(data)
-	}
-	return resourceSpec, nil
+	// we already validated the existence of files during validate()
+	return loadPropertyFile(strings.Replace(value, "file:", "", 1))
+}
+
+func keyValueProps(value string) (*properties.Properties, error) {
+	return properties.Load([]byte(value), properties.UTF8)
 }
 
 func (o *runCmdOptions) GetIntegrationName(sources []string) string {
@@ -680,24 +728,16 @@ func (o *runCmdOptions) configureTraits(integration *v1.Integration, options []s
 	return nil
 }
 
-func isLocalAndFileExists(fileName string) bool {
-	info, err := os.Stat(fileName)
-	if os.IsNotExist(err) {
-		return false
-	}
-	return !info.IsDir()
-}
-
-func addPropertyFile(fileName string, spec *v1.IntegrationSpec) error {
-	props, err := loadPropertyFile(fileName)
-	if err != nil {
-		return err
-	}
+func addIntegrationProperties(props *properties.Properties, spec *v1.IntegrationSpec) error {
 	for _, k := range props.Keys() {
 		v, _ := props.Get(k)
+		entry, err := property.EncodePropertyFileEntry(k, v)
+		if err != nil {
+			return err
+		}
 		spec.AddConfiguration(
 			"property",
-			escapePropertyFileItem(k)+"="+escapePropertyFileItem(v),
+			entry,
 		)
 	}
 	return nil
@@ -715,10 +755,35 @@ func loadPropertyFile(fileName string) (*properties.Properties, error) {
 	return p, nil
 }
 
-func escapePropertyFileItem(item string) string {
-	item = strings.ReplaceAll(item, `=`, `\=`)
-	item = strings.ReplaceAll(item, `:`, `\:`)
-	return item
+func resolvePodTemplate(ctx context.Context, templateSrc string, spec *v1.IntegrationSpec) (err error) {
+	// check if template is set
+	if templateSrc == "" {
+		return nil
+	}
+	var template v1.PodSpec
+
+	// check if value is a path to the file
+	if _, err := os.Stat(templateSrc); err == nil {
+		rsc, err := ResolveSources(ctx, []string{templateSrc}, false)
+		if err == nil && len(rsc) > 0 {
+			templateSrc = rsc[0].Content
+		}
+	}
+	// template is inline
+	templateBytes := []byte(templateSrc)
+
+	jsonTemplate, err := yaml.ToJSON(templateBytes)
+	if err != nil {
+		jsonTemplate = templateBytes
+	}
+	err = json.Unmarshal(jsonTemplate, &template)
+
+	if err == nil {
+		spec.PodTemplate = &v1.PodSpecTemplate{
+			Spec: template,
+		}
+	}
+	return err
 }
 
 func configureTraits(options []string, catalog *trait.Catalog) (map[string]v1.TraitSpec, error) {
