@@ -18,6 +18,7 @@ limitations under the License.
 package cmd
 
 import (
+	"context"
 	"io/ioutil"
 	"os"
 	"testing"
@@ -153,18 +154,6 @@ func TestRunLabelWrongFormatFlag(t *testing.T) {
 	assert.NotNil(t, err)
 }
 
-func TestRunLoggingLevelFlag(t *testing.T) {
-	runCmdOptions, rootCmd, _ := initializeRunCmdOptions(t)
-	_, err := test.ExecuteCommand(rootCmd, cmdRun,
-		"--logging-level", "lev1",
-		"--logging-level", "lev2",
-		integrationSource)
-	assert.Nil(t, err)
-	assert.Len(t, runCmdOptions.LoggingLevels, 2)
-	assert.Equal(t, "lev1", runCmdOptions.LoggingLevels[0])
-	assert.Equal(t, "lev2", runCmdOptions.LoggingLevels[1])
-}
-
 func TestRunLogsFlag(t *testing.T) {
 	runCmdOptions, rootCmd, _ := initializeRunCmdOptions(t)
 	_, err := test.ExecuteCommand(rootCmd, cmdRun, "--logs", integrationSource)
@@ -244,10 +233,13 @@ func TestRunPropertyFileFlagMissingFileExtension(t *testing.T) {
 
 const TestPropertyFileContent = `
 a=b
-c\=d=e
-d=c\=e
+# There's an issue in the properties lib: https://github.com/magiconair/properties/issues/59
+# so the following cases have been commented. Btw, we don't use equal sign in keys
+#c\=d=e
+#d=c\=e
 #ignore=me
-f=g\:h
+f=g:h
+i=j\nk
 `
 
 func TestAddPropertyFile(t *testing.T) {
@@ -261,12 +253,15 @@ func TestAddPropertyFile(t *testing.T) {
 	assert.Nil(t, ioutil.WriteFile(tmpFile.Name(), []byte(TestPropertyFileContent), 0644))
 
 	spec := v1.IntegrationSpec{}
-	assert.Nil(t, addPropertyFile(tmpFile.Name(), &spec))
-	assert.Equal(t, 4, len(spec.Configuration))
-	assert.Equal(t, `a=b`, spec.Configuration[0].Value)
-	assert.Equal(t, `c\=d=e`, spec.Configuration[1].Value)
-	assert.Equal(t, `d=c\=e`, spec.Configuration[2].Value)
-	assert.Equal(t, `f=g\:h`, spec.Configuration[3].Value)
+	properties, err := extractProperties("file:" + tmpFile.Name())
+	assert.Nil(t, err)
+	assert.Nil(t, addIntegrationProperties(properties, &spec))
+	assert.Equal(t, 3, len(spec.Configuration))
+	assert.Equal(t, `a = b`, spec.Configuration[0].Value)
+	//assert.Equal(t, `c\=d = e`, spec.Configuration[1].Value)
+	//assert.Equal(t, `d = c\=e`, spec.Configuration[2].Value)
+	assert.Equal(t, `f = g:h`, spec.Configuration[1].Value)
+	assert.Equal(t, `i = j\nk`, spec.Configuration[2].Value)
 }
 
 func TestRunPropertyFileFlag(t *testing.T) {
@@ -288,6 +283,15 @@ func TestRunPropertyFileFlag(t *testing.T) {
 	assert.Equal(t, tmpFile.Name(), runCmdOptions.PropertyFiles[0])
 }
 
+func TestRunProperty(t *testing.T) {
+	spec := v1.IntegrationSpec{}
+	properties, err := extractProperties(`key=value\nnewline`)
+	assert.Nil(t, err)
+	assert.Nil(t, addIntegrationProperties(properties, &spec))
+	assert.Equal(t, 1, len(spec.Configuration))
+	assert.Equal(t, `key = value\nnewline`, spec.Configuration[0].Value)
+}
+
 func TestRunResourceFlag(t *testing.T) {
 	runCmdOptions, rootCmd, _ := initializeRunCmdOptions(t)
 	_, err := test.ExecuteCommand(rootCmd, cmdRun,
@@ -408,6 +412,18 @@ func TestRunVolumeFlagWrongPVCFormat(t *testing.T) {
 	assert.NotNil(t, err)
 }
 
+func TestRunBuildPropertyFlag(t *testing.T) {
+	runCmdOptions, rootCmd, _ := initializeRunCmdOptions(t)
+	_, err := test.ExecuteCommand(rootCmd, cmdRun,
+		"--build-property", "build-prop1=val1",
+		"--build-property", "build-prop2=val2",
+		integrationSource)
+	assert.Nil(t, err)
+	assert.Len(t, runCmdOptions.BuildProperties, 2)
+	assert.Equal(t, "build-prop1=val1", runCmdOptions.BuildProperties[0])
+	assert.Equal(t, "build-prop2=val2", runCmdOptions.BuildProperties[1])
+}
+
 func TestRunValidateArgs(t *testing.T) {
 	runCmdOptions, rootCmd, _ := initializeRunCmdOptions(t)
 	args := []string{}
@@ -471,7 +487,7 @@ func TestRunWithSavedValues(t *testing.T) {
 }*/
 
 func TestRunBinaryResource(t *testing.T) {
-	binaryResourceSpec, err := binaryOrTextResource("file.ext", []byte{1, 2, 3, 4}, "application/octet-stream", false)
+	binaryResourceSpec, err := binaryOrTextResource("file.ext", []byte{1, 2, 3, 4}, "application/octet-stream", false, v1.ResourceTypeData, "")
 	assert.Nil(t, err)
 	assert.Equal(t, "", binaryResourceSpec.Content)
 	assert.NotNil(t, binaryResourceSpec.RawContent)
@@ -483,7 +499,7 @@ func TestRunBinaryResource(t *testing.T) {
 func TestRunBinaryCompressedResource(t *testing.T) {
 	data := []byte{1, 2, 3, 4}
 	base64Compressed, _ := compressToString(data)
-	binaryResourceSpec, err := binaryOrTextResource("file.ext", data, "application/octet-stream", true)
+	binaryResourceSpec, err := binaryOrTextResource("file.ext", data, "application/octet-stream", true, v1.ResourceTypeData, "")
 	assert.Nil(t, err)
 	assert.Equal(t, base64Compressed, binaryResourceSpec.Content)
 	assert.Nil(t, binaryResourceSpec.RawContent)
@@ -493,7 +509,7 @@ func TestRunBinaryCompressedResource(t *testing.T) {
 }
 
 func TestRunTextResource(t *testing.T) {
-	textResourceSpec, err := binaryOrTextResource("file.ext", []byte("hello world"), "text/plain", false)
+	textResourceSpec, err := binaryOrTextResource("file.ext", []byte("hello world"), "text/plain", false, v1.ResourceTypeData, "")
 	assert.Nil(t, err)
 	assert.Equal(t, "hello world", textResourceSpec.Content)
 	assert.Nil(t, textResourceSpec.RawContent)
@@ -505,7 +521,7 @@ func TestRunTextResource(t *testing.T) {
 func TestRunTextCompressedResource(t *testing.T) {
 	data := []byte("hello horld")
 	base64Compressed, _ := compressToString(data)
-	textResourceSpec, err := binaryOrTextResource("file.ext", []byte("hello horld"), "text/plain", true)
+	textResourceSpec, err := binaryOrTextResource("file.ext", []byte("hello horld"), "text/plain", true, v1.ResourceTypeData, "")
 	assert.Nil(t, err)
 	assert.Equal(t, base64Compressed, textResourceSpec.Content)
 	assert.Nil(t, textResourceSpec.RawContent)
@@ -514,9 +530,80 @@ func TestRunTextCompressedResource(t *testing.T) {
 	assert.True(t, textResourceSpec.Compression)
 }
 
-func TestIsLocalFileAndExists(t *testing.T) {
-	value, err := isLocalAndFileExists("/root/test")
-	// must not panic because a permission error
-	assert.NotNil(t, err)
-	assert.False(t, value)
+func TestResolvePodTemplate(t *testing.T) {
+	//runCmdOptions, rootCmd, _ := initializeRunCmdOptions(t)
+	templateText := `
+containers:
+  - name: integration
+    env:
+      - name: TEST
+        value: TEST
+    volumeMounts:
+      - name: var-logs
+        mountPath: /var/log
+volumes:
+  - name: var-logs
+    emptyDir: { }
+`
+
+	integrationSpec := v1.IntegrationSpec{}
+	err := resolvePodTemplate(context.TODO(), templateText, &integrationSpec)
+	assert.Nil(t, err)
+	assert.NotNil(t, integrationSpec.PodTemplate)
+	assert.Equal(t, 1, len(integrationSpec.PodTemplate.Spec.Containers))
+	//assert.Equal(t, 1,len(integrationSpec.PodTemplate.Spec.Containers[0].VolumeMounts))
+}
+
+func TestResolveJsonPodTemplate(t *testing.T) {
+	integrationSpec := v1.IntegrationSpec{}
+	minifiedYamlTemplate := `{"containers": [{"name": "second"}, {"name": "integration", "env": [{"name": "CAMEL_K_DIGEST", "value": "new_value"}]}]}`
+
+	err := resolvePodTemplate(context.TODO(), minifiedYamlTemplate, &integrationSpec)
+
+	assert.Nil(t, err)
+	assert.NotNil(t, integrationSpec.PodTemplate)
+	assert.Equal(t, 2, len(integrationSpec.PodTemplate.Spec.Containers))
+}
+
+func TestExtractProperties_SingleKeyValue(t *testing.T) {
+	correctValues := []string{"key=val", "key = val", "key= val", " key   =  val"}
+	for _, val := range correctValues {
+		prop, err := extractProperties(val)
+		assert.Nil(t, err)
+		value, ok := prop.Get("key")
+		assert.True(t, ok)
+		assert.Equal(t, "val", value)
+	}
+}
+
+func TestExtractProperties_FromFile(t *testing.T) {
+	var tmpFile1 *os.File
+	var err error
+	if tmpFile1, err = ioutil.TempFile("", "camel-k-*.properties"); err != nil {
+		t.Error(err)
+	}
+
+	assert.Nil(t, tmpFile1.Close())
+	assert.Nil(t, ioutil.WriteFile(tmpFile1.Name(), []byte(`
+	key=value
+	#key2=value2
+	my.key=value
+	`), 0644))
+
+	props, err := extractProperties("file:" + tmpFile1.Name())
+	assert.Nil(t, err)
+	assert.Equal(t, 2, props.Len())
+	for _, prop := range props.Keys() {
+		value, ok := props.Get(prop)
+		assert.True(t, ok)
+		assert.Equal(t, "value", value)
+	}
+}
+
+func TestFilterBuildPropertyFiles(t *testing.T) {
+	inputValues := []string{"file:/tmp/test", "key=val"}
+	outputValues := filterBuildPropertyFiles(inputValues)
+
+	assert.Equal(t, len(outputValues), 1)
+	assert.Equal(t, outputValues[0], "/tmp/test")
 }
