@@ -18,7 +18,9 @@ limitations under the License.
 package trait
 
 import (
+	"errors"
 	"fmt"
+	"github.com/apache/camel-k/pkg/util/source"
 	"regexp"
 	"sort"
 	"strconv"
@@ -32,7 +34,7 @@ import (
 	"github.com/apache/camel-k/pkg/platform"
 	"github.com/apache/camel-k/pkg/util"
 	"github.com/apache/camel-k/pkg/util/digest"
-	"github.com/apache/camel-k/pkg/util/flow"
+	"github.com/apache/camel-k/pkg/util/dsl"
 	"github.com/apache/camel-k/pkg/util/kubernetes"
 	corev1 "k8s.io/api/core/v1"
 	metav1 "k8s.io/apimachinery/pkg/apis/meta/v1"
@@ -94,28 +96,35 @@ func (t *kameletsTrait) Configure(e *Environment) (bool, error) {
 	}
 
 	if t.Auto == nil || *t.Auto {
+		var kamelets []string
 		if t.List == "" {
-			var kamelets []string
 			sources, err := kubernetes.ResolveIntegrationSources(e.C, e.Client, e.Integration, e.Resources)
 			if err != nil {
 				return false, err
 			}
 			metadata.Each(e.CamelCatalog, sources, func(_ int, meta metadata.IntegrationMetadata) bool {
-				util.StringSliceUniqueConcat(&kamelets, extractKamelets(meta.FromURIs))
-				util.StringSliceUniqueConcat(&kamelets, extractKamelets(meta.ToURIs))
+				util.StringSliceUniqueConcat(&kamelets, meta.Kamelets)
 				return true
 			})
+		}
+		// Check if a Kamelet is configured as default error handler URI
+		defaultErrorHandlerURI := e.Integration.Spec.GetConfigurationProperty(v1alpha1.ErrorHandlerAppPropertiesPrefix + ".deadLetterUri")
+		if defaultErrorHandlerURI != "" {
+			if strings.HasPrefix(defaultErrorHandlerURI, "kamelet:") {
+				kamelets = append(kamelets, source.ExtractKamelet(defaultErrorHandlerURI))
+			}
+		}
+
+		if len(kamelets) > 0 {
 			sort.Strings(kamelets)
 			t.List = strings.Join(kamelets, ",")
 		}
-
 	}
 
 	return len(t.getKameletKeys()) > 0, nil
 }
 
 func (t *kameletsTrait) Apply(e *Environment) error {
-
 	if e.IntegrationInPhase(v1.IntegrationPhaseInitialization, v1.IntegrationPhaseRunning) {
 		if err := t.addKamelets(e); err != nil {
 			return err
@@ -131,30 +140,76 @@ func (t *kameletsTrait) Apply(e *Environment) error {
 	return nil
 }
 
+func (t *kameletsTrait) collectKamelets(e *Environment) (map[string]*v1alpha1.Kamelet, error) {
+	repo, err := repository.NewForPlatform(e.C, e.Client, e.Platform, e.Integration.Namespace, platform.GetOperatorNamespace())
+	if err != nil {
+		return nil, err
+	}
+
+	kamelets := make(map[string]*v1alpha1.Kamelet)
+	missingKamelets := make([]string, 0)
+	availableKamelets := make([]string, 0)
+
+	for _, key := range t.getKameletKeys() {
+		kamelet, err := repo.Get(e.C, key)
+		if err != nil {
+			return nil, err
+		}
+
+		if kamelet == nil {
+			missingKamelets = append(missingKamelets, key)
+		} else {
+			availableKamelets = append(availableKamelets, key)
+
+			// Initialize remote kamelets
+			kamelets[key], err = kameletutils.Initialize(kamelet)
+			if err != nil {
+				return nil, err
+			}
+		}
+	}
+
+	sort.Strings(availableKamelets)
+	sort.Strings(missingKamelets)
+
+	if len(missingKamelets) > 0 {
+		message := fmt.Sprintf("kamelets %s found, %s not found in repositories: %s",
+			strings.Join(availableKamelets, ","),
+			strings.Join(missingKamelets, ","),
+			repo.String())
+
+		e.Integration.Status.SetCondition(
+			v1.IntegrationConditionKameletsAvailable,
+			corev1.ConditionFalse,
+			v1.IntegrationConditionKameletsAvailableReason,
+			message,
+		)
+
+		return nil, errors.New(message)
+	}
+
+	e.Integration.Status.SetCondition(
+		v1.IntegrationConditionKameletsAvailable,
+		corev1.ConditionTrue,
+		v1.IntegrationConditionKameletsAvailableReason,
+		fmt.Sprintf("kamelets %s found in repositories: %s", strings.Join(availableKamelets, ","), repo.String()),
+	)
+
+	return kamelets, nil
+}
+
 func (t *kameletsTrait) addKamelets(e *Environment) error {
-	kameletKeys := t.getKameletKeys()
-	if len(kameletKeys) > 0 {
-		repo, err := repository.NewForPlatform(e.C, e.Client, e.Platform, e.Integration.Namespace, platform.GetOperatorNamespace())
+	if len(t.getKameletKeys()) > 0 {
+		kamelets, err := t.collectKamelets(e)
 		if err != nil {
 			return err
 		}
-		for _, k := range t.getKameletKeys() {
-			kamelet, err := repo.Get(e.C, k)
-			if err != nil {
-				return err
-			}
-			if kamelet == nil {
-				return fmt.Errorf("kamelet %s not found in any of the defined repositories: %s", k, repo.String())
-			}
 
-			// Initialize remote kamelets
-			kamelet, err = kameletutils.Initialize(kamelet)
-			if err != nil {
-				return err
-			}
+		for _, key := range t.getKameletKeys() {
+			kamelet := kamelets[key]
 
 			if kamelet.Status.Phase != v1alpha1.KameletPhaseReady {
-				return fmt.Errorf("kamelet %q is not %s: %s", k, v1alpha1.KameletPhaseReady, kamelet.Status.Phase)
+				return fmt.Errorf("kamelet %q is not %s: %s", key, v1alpha1.KameletPhaseReady, kamelet.Status.Phase)
 			}
 
 			if err := t.addKameletAsSource(e, kamelet); err != nil {
@@ -172,25 +227,13 @@ func (t *kameletsTrait) addKamelets(e *Environment) error {
 
 func (t *kameletsTrait) configureApplicationProperties(e *Environment) error {
 	if len(t.getKameletKeys()) > 0 {
-		repo, err := repository.NewForPlatform(e.C, e.Client, e.Platform, e.Integration.Namespace, platform.GetOperatorNamespace())
+		kamelets, err := t.collectKamelets(e)
 		if err != nil {
 			return err
 		}
-		for _, k := range t.getKameletKeys() {
-			kamelet, err := repo.Get(e.C, k)
-			if err != nil {
-				return err
-			}
-			if kamelet == nil {
-				return fmt.Errorf("kamelet %s not found in any of the defined repositories: %s", k, repo.String())
-			}
-
-			// remote Kamelets may not be fully initialized
-			kamelet, err = kameletutils.Initialize(kamelet)
-			if err != nil {
-				return err
-			}
 
+		for _, key := range t.getKameletKeys() {
+			kamelet := kamelets[key]
 			// Configuring defaults from Kamelet
 			for _, prop := range kamelet.Status.Properties {
 				if prop.Default != "" {
@@ -217,28 +260,27 @@ func (t *kameletsTrait) configureApplicationProperties(e *Environment) error {
 func (t *kameletsTrait) addKameletAsSource(e *Environment, kamelet *v1alpha1.Kamelet) error {
 	sources := make([]v1.SourceSpec, 0)
 
-	if kamelet.Spec.Flow != nil {
-
-		flowData, err := flow.ToYamlDSL([]v1.Flow{*kamelet.Spec.Flow})
+	if kamelet.Spec.Template != nil || kamelet.Spec.Flow != nil {
+		template := kamelet.Spec.Template
+		if template == nil {
+			// Backward compatibility with Kamelets using flow
+			template = &v1.Template{
+				RawMessage: kamelet.Spec.Flow.RawMessage,
+			}
+		}
+		flowData, err := dsl.TemplateToYamlDSL(*template, kamelet.Name)
 		if err != nil {
 			return err
 		}
 
-		propertyNames := make([]string, 0, len(kamelet.Status.Properties))
-		for _, p := range kamelet.Status.Properties {
-			propertyNames = append(propertyNames, p.Name)
-		}
-
 		flowSource := v1.SourceSpec{
 			DataSpec: v1.DataSpec{
 				Name:    fmt.Sprintf("%s.yaml", kamelet.Name),
 				Content: string(flowData),
 			},
-			Language:      v1.LanguageYaml,
-			Type:          v1.SourceTypeTemplate,
-			PropertyNames: propertyNames,
+			Language: v1.LanguageYaml,
 		}
-		flowSource, err = integrationSourceFromKameletSource(e, kamelet, flowSource, fmt.Sprintf("%s-kamelet-%s-flow", e.Integration.Name, kamelet.Name))
+		flowSource, err = integrationSourceFromKameletSource(e, kamelet, flowSource, fmt.Sprintf("%s-kamelet-%s-template", e.Integration.Name, kamelet.Name))
 		if err != nil {
 			return err
 		}
@@ -253,11 +295,7 @@ func (t *kameletsTrait) addKameletAsSource(e *Environment, kamelet *v1alpha1.Kam
 		sources = append(sources, intSource)
 	}
 
-	kameletCounter := 0
 	for _, source := range sources {
-		if source.Type == v1.SourceTypeTemplate {
-			kameletCounter++
-		}
 		replaced := false
 		for idx, existing := range e.Integration.Status.GeneratedSources {
 			if existing.Name == source.Name {
@@ -270,10 +308,6 @@ func (t *kameletsTrait) addKameletAsSource(e *Environment, kamelet *v1alpha1.Kam
 		}
 	}
 
-	if kameletCounter > 1 {
-		return fmt.Errorf(`kamelet %s contains %d sources of type "kamelet": at most one is allowed`, kamelet.Name, kameletCounter)
-	}
-
 	return nil
 }
 
@@ -401,13 +435,3 @@ func integrationSourceFromKameletSource(e *Environment, kamelet *v1alpha1.Kamele
 	target.ContentKey = contentKey
 	return *target, nil
 }
-
-func extractKamelets(uris []string) (kamelets []string) {
-	for _, uri := range uris {
-		matches := kameletNameRegexp.FindStringSubmatch(uri)
-		if len(matches) > 1 {
-			kamelets = append(kamelets, matches[1])
-		}
-	}
-	return
-}
