@@ -35,7 +35,7 @@ func TestRuntimeContainsEmbeddedArtifacts(t *testing.T) {
 	artifact := catalog.GetArtifactByScheme("knative")
 	assert.Equal(t, 1, len(artifact.Schemes))
 	assert.Equal(t, "org.apache.camel.k", artifact.GroupID)
-	assert.Equal(t, "camel-knative", artifact.ArtifactID)
+	assert.Equal(t, "camel-k-knative", artifact.ArtifactID)
 
 	scheme, found := catalog.GetScheme("knative")
 	assert.True(t, found)
